@@ -102,7 +102,6 @@ JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
 using namespace juce;
 
-#include "../utility/juce_FakeMouseMoveGenerator.h"
 #include "../utility/juce_WindowsHooks.h"
 #include "../utility/juce_LinuxMessageThread.h"
 
@@ -447,7 +446,7 @@ public:
                     const int numChannels = jmax (numIn, numOut);
                     AudioBuffer<FloatType> chans (tmpBuffers.channels, isMidiEffect ? 0 : numChannels, numSamples);
 
-                    if (isBypassed)
+                    if (isBypassed && processor->getBypassParameter() == nullptr)
                         processor->processBlockBypassed (chans, midiEvents);
                     else
                         processor->processBlock (chans, midiEvents);
@@ -633,39 +632,39 @@ public:
         info.ppqPosition = (ti->flags & Vst2::kVstPpqPosValid) != 0 ? ti->ppqPos : 0.0;
         info.ppqPositionOfLastBarStart = (ti->flags & Vst2::kVstBarsValid) != 0 ? ti->barStartPos : 0.0;
 
-        if ((ti->flags & Vst2::kVstSmpteValid) != 0)
+        std::tie (info.frameRate, info.editOriginTime) = [ti]
         {
-            AudioPlayHead::FrameRateType rate = AudioPlayHead::fpsUnknown;
-            double fps = 1.0;
+            if ((ti->flags & Vst2::kVstSmpteValid) == 0)
+                return std::make_tuple (FrameRate(), 0.0);
 
-            switch (ti->smpteFrameRate)
+            const auto rate = [&]
             {
-                case Vst2::kVstSmpte239fps:       rate = AudioPlayHead::fps23976;    fps = 24.0 * 1000.0 / 1001.0; break;
-                case Vst2::kVstSmpte24fps:        rate = AudioPlayHead::fps24;       fps = 24.0;  break;
-                case Vst2::kVstSmpte25fps:        rate = AudioPlayHead::fps25;       fps = 25.0;  break;
-                case Vst2::kVstSmpte2997fps:      rate = AudioPlayHead::fps2997;     fps = 30.0 * 1000.0 / 1001.0; break;
-                case Vst2::kVstSmpte30fps:        rate = AudioPlayHead::fps30;       fps = 30.0;  break;
-                case Vst2::kVstSmpte2997dfps:     rate = AudioPlayHead::fps2997drop; fps = 30.0 * 1000.0 / 1001.0; break;
-                case Vst2::kVstSmpte30dfps:       rate = AudioPlayHead::fps30drop;   fps = 30.0;  break;
+                switch (ti->smpteFrameRate)
+                {
+                    case Vst2::kVstSmpte24fps:          return FrameRate().withBaseRate (24);
+                    case Vst2::kVstSmpte239fps:         return FrameRate().withBaseRate (24).withPullDown();
 
-                case Vst2::kVstSmpteFilm16mm:
-                case Vst2::kVstSmpteFilm35mm:     fps = 24.0; break;
+                    case Vst2::kVstSmpte25fps:          return FrameRate().withBaseRate (25);
+                    case Vst2::kVstSmpte249fps:         return FrameRate().withBaseRate (25).withPullDown();
 
-                case Vst2::kVstSmpte249fps:       fps = 25.0 * 1000.0 / 1001.0; break;
-                case Vst2::kVstSmpte599fps:       fps = 60.0 * 1000.0 / 1001.0; break;
-                case Vst2::kVstSmpte60fps:        fps = 60; break;
+                    case Vst2::kVstSmpte30fps:          return FrameRate().withBaseRate (30);
+                    case Vst2::kVstSmpte30dfps:         return FrameRate().withBaseRate (30).withDrop();
+                    case Vst2::kVstSmpte2997fps:        return FrameRate().withBaseRate (30).withPullDown();
+                    case Vst2::kVstSmpte2997dfps:       return FrameRate().withBaseRate (30).withPullDown().withDrop();
 
-                default:                          jassertfalse; // unknown frame-rate..
-            }
+                    case Vst2::kVstSmpte60fps:          return FrameRate().withBaseRate (60);
+                    case Vst2::kVstSmpte599fps:         return FrameRate().withBaseRate (60).withPullDown();
 
-            info.frameRate = rate;
-            info.editOriginTime = ti->smpteOffset / (80.0 * fps);
-        }
-        else
-        {
-            info.frameRate = AudioPlayHead::fpsUnknown;
-            info.editOriginTime = 0;
-        }
+                    case Vst2::kVstSmpteFilm16mm:
+                    case Vst2::kVstSmpteFilm35mm:       return FrameRate().withBaseRate (24);
+                }
+
+                return FrameRate();
+            }();
+
+            const auto effectiveRate = rate.getEffectiveRate();
+            return std::make_tuple (rate, effectiveRate != 0.0 ? ti->smpteOffset / (80.0 * effectiveRate) : 0.0);
+        }();
 
         info.isRecording = (ti->flags & Vst2::kVstTransportRecording) != 0;
         info.isPlaying   = (ti->flags & (Vst2::kVstTransportRecording | Vst2::kVstTransportPlaying)) != 0;
@@ -702,12 +701,7 @@ public:
     void setParameter (int32 index, float value)
     {
         if (auto* param = juceParameters.getParamForIndex (index))
-        {
-            param->setValue (value);
-
-            inParameterChangedCallback = true;
-            param->sendValueChangedMessageToListeners (value);
-        }
+            setValueAndNotifyIfChanged (*param, value);
     }
 
     static void setParameterCB (Vst2::AEffect* vstInterface, int32 index, float value)
@@ -742,7 +736,7 @@ public:
     void parameterValueChanged (int, float newValue) override
     {
         // this can only come from the bypass parameter
-        isBypassed = (newValue != 0.0f);
+        isBypassed = (newValue >= 0.5f);
     }
 
     void parameterGestureChanged (int, bool) override {}
@@ -849,7 +843,7 @@ public:
             if (auto* ed = processor->createEditorIfNeeded())
             {
                 setHasEditorFlag (true);
-                editorComp.reset (new EditorCompWrapper (*this, *ed));
+                editorComp.reset (new EditorCompWrapper (*this, *ed, editorScaleFactor));
             }
             else
             {
@@ -973,10 +967,15 @@ public:
                               , public Timer
                              #endif
     {
-        EditorCompWrapper (JuceVSTWrapper& w, AudioProcessorEditor& editor)
+        EditorCompWrapper (JuceVSTWrapper& w, AudioProcessorEditor& editor, float initialScale)
             : wrapper (w)
         {
             editor.setOpaque (true);
+           #if ! JUCE_MAC
+            editor.setScaleFactor (initialScale);
+           #else
+            ignoreUnused (initialScale);
+           #endif
             addAndMakeVisible (editor);
 
             auto editorBounds = getSizeToContainChild();
@@ -988,7 +987,6 @@ public:
            #endif
 
             setOpaque (true);
-            ignoreUnused (fakeMouseGenerator);
         }
 
         ~EditorCompWrapper() override
@@ -1217,24 +1215,19 @@ public:
 
         void setContentScaleFactor (float scale)
         {
-            if (! approximatelyEqual (scale, editorScaleFactor))
+            if (auto* pluginEditor = getEditorComp())
             {
-                editorScaleFactor = scale;
+                auto prevEditorBounds = pluginEditor->getLocalArea (this, lastBounds);
 
-                if (auto* pluginEditor = getEditorComp())
                 {
-                    auto prevEditorBounds = pluginEditor->getLocalArea (this, lastBounds);
+                    const ScopedValueSetter<bool> resizingChildSetter (resizingChild, true);
 
-                    {
-                        const ScopedValueSetter<bool> resizingChildSetter (resizingChild, true);
-
-                        pluginEditor->setScaleFactor (editorScaleFactor);
-                        pluginEditor->setBounds (prevEditorBounds.withPosition (0, 0));
-                    }
-
-                    lastBounds = getSizeToContainChild();
-                    updateWindowSize();
+                    pluginEditor->setScaleFactor (scale);
+                    pluginEditor->setBounds (prevEditorBounds.withPosition (0, 0));
                 }
+
+                lastBounds = getSizeToContainChild();
+                updateWindowSize();
             }
         }
 
@@ -1258,7 +1251,7 @@ public:
          {
              auto hostWindowScale = (float) getScaleFactorForWindow ((HostWindowType) hostWindow);
 
-             if (hostWindowScale > 0.0f && ! approximatelyEqual (hostWindowScale, editorScaleFactor))
+             if (hostWindowScale > 0.0f && ! approximatelyEqual (hostWindowScale, wrapper.editorScaleFactor))
                  wrapper.handleSetContentScaleFactor (hostWindowScale);
          }
 
@@ -1295,10 +1288,8 @@ public:
 
         //==============================================================================
         JuceVSTWrapper& wrapper;
-        FakeMouseMoveGenerator fakeMouseGenerator;
         bool resizingChild = false, resizingParent = false;
 
-        float editorScaleFactor = 1.0f;
         juce::Rectangle<int> lastBounds;
 
        #if JUCE_LINUX || JUCE_BSD
@@ -1432,6 +1423,15 @@ private:
     static void checkWhetherMessageThreadIsCorrect() {}
    #endif
 
+    void setValueAndNotifyIfChanged (AudioProcessorParameter& param, float newValue)
+    {
+        if (param.getValue() == newValue)
+            return;
+
+        inParameterChangedCallback = true;
+        param.setValueNotifyingHost (newValue);
+    }
+
     //==============================================================================
     template <typename FloatType>
     void deleteTempChannels (VstTempBuffers<FloatType>& tmpBuffers)
@@ -1715,12 +1715,7 @@ private:
         {
             if (! LegacyAudioParameter::isLegacy (param))
             {
-                auto value = param->getValueForText (String::fromUTF8 ((char*) args.ptr));
-                param->setValue (value);
-
-                inParameterChangedCallback = true;
-                param->sendValueChangedMessageToListeners (value);
-
+                setValueAndNotifyIfChanged (*param, param->getValueForText (String::fromUTF8 ((char*) args.ptr)));
                 return 1;
             }
         }
@@ -1804,10 +1799,10 @@ private:
 
     pointer_sized_int handleSetBypass (VstOpCodeArguments args)
     {
-        isBypassed = (args.value != 0);
+        isBypassed = args.value != 0;
 
-        if (auto* bypass = processor->getBypassParameter())
-            bypass->setValueNotifyingHost (isBypassed ? 1.0f : 0.0f);
+        if (auto* param = processor->getBypassParameter())
+            param->setValueNotifyingHost (isBypassed ? 1.0f : 0.0f);
 
         return 1;
     }
@@ -1993,9 +1988,18 @@ private:
 
     pointer_sized_int handleSetContentScaleFactor (float scale)
     {
+        checkWhetherMessageThreadIsCorrect();
+        const MessageManagerLock mmLock;
+
        #if ! JUCE_MAC
-        if (editorComp != nullptr)
-            editorComp->setContentScaleFactor (scale);
+        if (! approximatelyEqual (scale, editorScaleFactor))
+        {
+            editorScaleFactor = scale;
+
+            if (editorComp != nullptr)
+                editorComp->setContentScaleFactor (editorScaleFactor);
+        }
+
        #else
         ignoreUnused (scale);
        #endif
@@ -2065,6 +2069,7 @@ private:
     CriticalSection stateInformationLock;
     juce::MemoryBlock chunkMemory;
     uint32 chunkMemoryTime = 0;
+    float editorScaleFactor = 1.0f;
     std::unique_ptr<EditorCompWrapper> editorComp;
     Vst2::ERect editorRect;
     MidiBuffer midiEvents;
