@@ -35,7 +35,7 @@
 
   ID:                 juce_box2d
   vendor:             juce
-  version:            6.1.2
+  version:            6.1.6
   name:               JUCE wrapper for the Box2D physics engine
   description:        The Box2D physics engine and some utility classes.
   website:            http://www.juce.com/juce
@@ -58,7 +58,8 @@
 JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wconversion",
                                      "-Wshadow-field",
                                      "-Wzero-as-null-pointer-constant",
-                                     "-Wsign-conversion")
+                                     "-Wsign-conversion",
+                                     "-Wdeprecated")
 
 #include <climits>
 #include <cfloat>
