@@ -35,7 +35,7 @@
 
   ID:                 juce_gui_basics
   vendor:             juce
-  version:            6.1.3
+  version:            6.1.6
   name:               JUCE GUI core classes
   description:        Basic user-interface components and related classes.
   website:            http://www.juce.com/juce
@@ -158,9 +158,11 @@ namespace juce
     class Displays;
     class AccessibilityHandler;
     class KeyboardFocusTraverser;
+    class PointerState;
 
     class FlexBox;
     class Grid;
+    class FocusOutline;
 
    #if JUCE_MAC || JUCE_WINDOWS || JUCE_LINUX
     Image createSnapshotOfNativeWindow (void* nativeWindowHandle);
@@ -259,6 +261,7 @@ namespace juce
 #include "menus/juce_BurgerMenuComponent.h"
 #include "buttons/juce_ToolbarButton.h"
 #include "misc/juce_DropShadower.h"
+#include "misc/juce_FocusOutline.h"
 #include "misc/juce_JUCESplashScreen.h"
 #include "widgets/juce_TreeView.h"
 #include "windows/juce_TopLevelWindow.h"
